# -----------------------------------------------------------------------------
sub sqlFG 
# -----------------------------------------------------------------------------
{
  # -----------------------------------------------------------------------------
  # Check SQL filegroups -- Similar to an Oracle tablespace
  # -----------------------------------------------------------------------------

  #---- Variables
  my $dg_pct = 0; 
  my $dg_overflow_mb = 0; 

  #----
  #---- This routine uses stacked data
  #----

  ($thishost) = @_;

  if ($t_fgwarn[$thishost] eq "N")
  {
    &logit("(Not Selected (T_FG_Full: is N)",$thishost);

    &logDetails($thishost,"<br><a name='sqlFG'> <h3>MSSQL Filegroup Status</h3>");
    &logDetails($thishost,"<h5>T_FG_Full: is N, so sqlFG() skipped</h5>");
    &logDetails($thishost,"</table><hr>");

    return 1;
  }

  $debug_fg = 0; 

  if ($up[$thishost] ne "Y")
  {
    &logit("(Down/Inhibited)",$thishost);
    return 1;
  }

  if (&check_inhib($thishost) eq "Y")
  {
    $up[$thishost] = "I";
    return;
  }

  #---- Defaults if T_FG_Full: is not specified
  $t_fgwarn[$thishost] = "80" if !$t_fgwarn[$thishost];
  $t_fgcrit[$thishost] = "90" if !$t_fgcrit[$thishost];

  #---- Start
  &logDetails($thishost,"</table>");
  &logDetails($thishost,"<br><a name='sqlFG'> <h3>MSSQL Filegroup Status</h3>");
  &logDetails($thishost,"<h5>Warning Threshold: ${t_fgwarn[$thishost]}% - Critical Threshold: ${t_fgcrit[$thishost]}% </h5>");
  &logDetails($thishost,"<table border=0 bgcolor=$table_bgcolor>");
  &logDetails($thishost,"<tr bgcolor=$table_headbgcolor>
    <th>$f_s DBName
    <th>$f_s Filegroup
    <th>$f_s Filegroup<br>Total Size<br>(MB)
    <th>$f_s Filegroup<br>Max. Size<br>(MB)
    <th>$f_s Filegroup<br>% Full<br>vs.<br>Max Size 
    <th>$f_s Filegroup<br>Growth Potential <br>(MB) 
    <th>$f_s Filegroup<br>Drive List 
  ");

  # 2011/01/05-10:44:07 dbamon_mssql.pl 2.00 | FG Final Results:
  # #Legend:                 DBName                                   FGName                TotalSizeMB        MaxSizeMB   PctFullvsMax
  # #Legend: (MaxSizeMB is -1 if any DFs are MAXUNLIMITED) PctFullvsMax
  # dbamon_mssql.pl FG_ITEM: bbtest1                                  PRIMARY                      2.25                0          -1.00
  # dbamon_mssql.pl FG_ITEM: dbamon                                   PRIMARY                     21.25              200          10.63

  foreach (split('\n',$fgdata[$thishost]))
  {
    ##print "Line: $_ \n"; 

    $event_fg_flag = 0; 

    my @words = split;
    my $fg_dbname         = $words[2]; 
    my $fg_fgname         = $words[3]; 
    my $fg_fgtotsize_mb   = $words[4]; 
    my $fg_fgmaxsize_mb   = $words[5]; 
    my $fg_pctfull_vs_max = $words[6]; 
    my $fg_potgrowth_mb   = $words[7]; 
    my $fg_drivelist      = $words[8]; 

    my $fg_pctfull_vs_max_int = int($fg_pctfull_vs_max); 

    print "DEBUG_FG1 - fg_dbname=$fg_dbname fg_fgname=$fg_fgname fg_pctfull_vs_max=$fg_pctfull_vs_max fg_potgrowth_mb=$fg_potgrowth_mb fg_drivelist=$fg_drivelist fg_fgmaxsize_mb=$fg_fgmaxsize_mb\n" if $debug_fg;
    &metricsInsert("sqlFG()",$thishost,"FG_PCT_FULL","$fg_dbname/$fg_fgname",$fg_pctfull_vs_max) if $fg_pctfull_vs_max > 0;

    #---- Decide whether this is an event
    if ($fg_pctfull_vs_max > 0) 
    {
      &metricsCurrent("sqlFG()",$thishost,"FILEGROUP_PCT_FULL",$fg_dbname,$fg_pctfull_vs_max,$t_fgwarn[$thishost],$t_fgcrit[$thishost],"FG % Full vs. MAX");

      if ($fg_pctfull_vs_max >= $t_fgcrit[$thishost]) 
      {
        $www_color = "pink"; 
        $ev_cat = "Space";
        &event("C",$thishost,"DB=$fg_dbname Filegroup=$fg_fgname - DB File(s) are ${fg_pctfull_vs_max_int}% Full vs. Filegroup Total MAXSIZE" ,"FG-Full-MAXSIZE","DBA850");
        $event_fg_flag = 1; 
      }
      elsif ($fg_pctfull_vs_max >= $t_fgwarn[$thishost]) 
      {
        $www_color = "yellow"; 
        $ev_cat = "Space";
        &event("W",$thishost,"DB=$fg_dbname Filegroup=$fg_fgname - DB File(s) are ${fg_pctfull_vs_max_int}% Full vs. Filegroup Total MAXSIZE" ,"FG-Full-MAXSIZE","DBA850");
        $event_fg_flag = 1; 
      }
      else
      {
        $www_color = "lime"; 
      }
    }
    else   #---- pctfull <= 0 - unlimited growth
    {
      $www_color = "white"; 
    }

    #---- WWW Content 
    if ($fg_fgmaxsize_mb < 0)   #---- The negative value here should only  happen for FIX SIZE (no growth) FGs
    {
      $zmaxsize = "(At least 1 DB File is Fixed Size (NoGrowth))"; 
      $zpotgrowth = "0"; 
      $zpctvsmax = "100"; 

      #---- Event DBA200 for FG at maxsize-unlimited
      $ev_cat = "Mgmt";
      &event("W",$thishost,"DBName=$fg_dbname FileGroup=$fg_fgname Has At Least 1 File is Fixed Size (NoGrowth)","FG-DF-FixedSize","DBA200");
    }
    elsif ($fg_fgmaxsize_mb == 0 && $fg_drivelist eq "")
    {
      $zmaxsize = "(This Filegroup has NO files)"; 
      $zpotgrowth = "(N/A)"; 
      $zpctvsmax = "(N/A)"; 
    }
    elsif ($fg_fgmaxsize_mb == 0)
    {
      $zmaxsize = "(At least 1 DB File is MaxSize Unlimited)"; 
      $zpotgrowth = "(N/A)"; 
      $zpctvsmax = "(N/A)"; 

      #---- Event DBA200 for FG at maxsize-unlimited
      $ev_cat = "Mgmt";
      &event("U",$thishost,"DBName=$fg_dbname FileGroup=$fg_fgname Has At Least 1 File With MaxSize=Unlimited","FG-DF-Unlimited","DBA200");
    }
    else
    {
      $zmaxsize = $fg_fgmaxsize_mb; 
      $zpotgrowth = $fg_potgrowth_mb; 
      $zpctvsmax = "${fg_pctfull_vs_max}%"; 
    }

    $z = "<tr bgcolor=$www_color> 
      <td>$f_s <b> $fg_dbname </b>
      <td>$f_s <b> $fg_fgname </b>
      <td align=right>$f_s $fg_fgtotsize_mb
      <td align=right>$f_s $zmaxsize
      <td align=right>$f_s <b> $zpctvsmax </b>
      <td align=right>$f_s $zpotgrowth
      <td >$f_s $fg_drivelist
    ";
    &logDetails($thishost,$z);


    if ($event_fg_flag == 0)
    {
      &logit("OK - DB=($fg_dbname) fg_fgname=$fg_fgname - fg_pctfull_vs_max=$fg_pctfull_vs_max fg_fgtotsize_mb=$fg_fgtotsize_mb fg_fgmaxsize_mb=$fg_fgmaxsize_mb fg_potgrowth_mb=$fg_potgrowth_mb",$thishost);
    }
  }

  &logDetails($thishost,"</table><hr>");

  #
  #---- Now, process the DRIVE data
  #

  $t_dg_critical = 100;
  $t_dg_warning  = 98;

  &logDetails($thishost,"</table>");
  &logDetails($thishost,"<br><a name='sqlFGDRIVE'> <h3>Drive Capacity Status - DB File Growth Potential vs. Drive Freespace </h3>");
  &logDetails($thishost,"<h5>Warning Threshold: ${t_dg_warning}% - Critical Threshold: ${t_dg_critical}% </h5>");
  &logDetails($thishost,"<table border=0 bgcolor=$table_bgcolor>");
  &logDetails($thishost,"<tr bgcolor=$table_headbgcolor>
    <th>$f_s Drive
    <th>$f_s Potential Growth<br>of All DB Files<br>Using This Drive<br>(MB)
    <th>$f_s Drive Freespace <br>(MB)
    <th>$f_s Potential<br> vs. <br> Freespace<br> Percent 
  ");

  # #Legend:                 Drive     PotGrowthMB     FreeMB
  # dbamon_mssql.pl FG_DRIVE_GROWTH: C 178.75 55887.00

  foreach (split('\n',$drivegrowthdata[$thishost]))
  {
    ##print "Line: $_ \n"; 

    $event_dg_flag = 0; 

    @words = split;
    $dg_drive          = $words[2]; 
    $dg_potgrowth_mb   = $words[3]; 
    $dg_drivefree_mb   = $words[4]; 

    if ($dg_drivefree_mb <= 0) 
    {
      &error_msg("DG - $titleraw[$thishost] - dg_drivefree_mb=($dg_drivefree_mb) <= 0 - dg_drive=$dg_drive line=($_)"); 
      next;
    }

    $dg_pct         = sprintf("%.2f",(($dg_potgrowth_mb / $dg_drivefree_mb) * 100)); 
    $dg_overflow_mb = sprintf("%.2f",(($dg_potgrowth_mb - $dg_drivefree_mb) + 10)); 

    print "DEBUG_dg_1 - dr_drive=$dr_drive dg_potgrowth_mb=$dg_potgrowth_mb dg_drivefree_mb=$dg_drivefree_mb dg_pct=$dg_pct\n" if $debug_fg;

    #---- Decide whether this is an event

    &metricsCurrent("sqlFG()",$thishost,"DISK_FG_GROWTH_POTENTIAL",$dg_drive,$dg_pct,$t_dg_warning,$t_dg_warning,
      "DB Files on drive $dg_drive have the potential to grow $dg_potgrowth_mb - This represents ${dg_pct}% of the $dg_drivefree_mb (MB) of freespace on this drive");

    #---- Prepare event file just in case
    open(TMPFL2, ">$tmpfl2") || &error_msg("Can't open output $tmpfl2 - $!");
    print TMPFL2 "Drive:                                                 $dg_drive\n";
    print TMPFL2 "(A) Potential Growth of All DB Files Using This Drive: $dg_potgrowth_mb (MB) - Versus DB File MaxSize\n";
    print TMPFL2 "(B) Drive Freespace:                                   $dg_drivefree_mb (MB)\n";
    print TMPFL2 "(A) Divided By (B):                                    $dg_pct (%)\n";
    print TMPFL2 "Critical Threshold:                                    $t_dg_critical (%)\n";
    print TMPFL2 "Warning Threshold:                                     $t_dg_warning (%)\n";
    print TMPFL2 "(A) Minus (B) Overflow Amount:                         $dg_overflow_mb (MB) - Decrease DB File Maxsize(s) by this amount to clear this event\n";
    print TMPFL2 "\nGo to: <a href='$html_details_url[$thishost]'><b>DBAmon Instance Detail (Drill Down) Page</b></a> for Drive and FileGroup Details.\n";
    close(TMPFL2);

    $dg_pct_int = int($dg_pct);
    $dg_potgrowth_gb = int($dg_potgrowth_mb / 1024); 

    if ($dg_pct >= $t_dg_critical) 
    {
      $www_color = "pink"; 
      ##&event("C",$thishost,"DB Files On Drive ${dg_drive}: Have the Potential to Grow: $dg_potgrowth_gb(GB) Which is >= Drive Freespace - % of Freespace: ${dg_pct_int}% (Critical Threshold=${t_dg_critical}%)" ,"DriveGrowth-Overflow","DBA851",$tmpfl2);
      $ev_cat = "Space";
      &event("C",$thishost,"DB Files On Drive ${dg_drive}: Have the Potential to Grow Beyond Drive Freespace" ,"Drive-${dg_drive}-Overflow","DBA851",$tmpfl2);
      $event_dg_flag = 1; 
    }
    elsif ($dg_pct >= $t_dg_warning) 
    {
      $www_color = "yellow"; 
      ##&event("W",$thishost,"DB Files On Drive ${dg_drive}: Have the Potential to Grow: $dg_potgrowth_gb(MB) Which is >= Drive Freespace - % of Freespace: ${dg_pct_int}% (Warning Threshold=${t_dg_critical}%)" ,"DriveGrowth-Overflow","DBA851",$tmpfl2);
      $ev_cat = "Space";
      &event("W",$thishost,"DB Files On Drive ${dg_drive}: Have the Potential to Grow Beyond Drive Freespace" ,"Drive-${dg_drive}-Overflow","DBA851",$tmpfl2);
      $event_dg_flag = 1; 
    }
    else
    {
      $www_color = "lime"; 
    }

    $zpot  = &commify($dg_potgrowth_mb); 
    $zfree = &commify($dg_drivefree_mb); 
 
    $z = "<tr bgcolor=$www_color> 
      <td>$f_s <b> ${dg_drive}: </b>
      <td align=right>$f_s $zpot
      <td align=right>$f_s $zfree
      <td align=right>$f_s <b> ${dg_pct}% </b>
    ";
    &logDetails($thishost,$z);

    &metricsInsert("sqlFG()",$thishost,"DISK_POTG_PCT","${dg_drive}",$dg_pct); 

    if ($event_dg_flag == 0)
    {
      &logit("OK - Drive=($dg_drive) dg_pct=$dg_pct - dg_potgrowth_mb=$dg_potgrowth_mb dg_drivefree_mb=$dg_drivefree_mb",$thishost);
    }
  }

  &logDetails($thishost,"</table><hr>");

}
# -----------------------------------------------------------------------------
1;
